#define _GNU_SOURCE

#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "util.h"
#include "conf.h"

unsigned short		cfg_queue = 4444;
unsigned int		cfg_bad_entries = 4096;
char			*cfg_sock_path = "/etc/nf2fa.sock";
unsigned int		cfg_grant_time = 3600;
unsigned char		cfg_min_pass_len = 0;
unsigned int		cfg_nfmark = 0x77777777;
unsigned int		cfg_ctmark = 0x77777777;


/*
 * Log configuration options
 */
static void log_conf(void)
{
	xlog("queue: %u\n", cfg_queue);
	xlog("bad entries: %u\n", cfg_bad_entries);
	xlog("sock: %s\n", cfg_sock_path);
	xlog("grant time: %u\n", cfg_grant_time);
	xlog("min pass len: %u\n", cfg_min_pass_len);
	xlog("nfmark: 0x%08x\n", cfg_nfmark);
	xlog("ctmark: 0x%08x\n", cfg_ctmark);
}

/*
 * Loading configuration
 * Return -1 on error, 0 on success
 */
int conf_load(const char *file)
{
	int err = -1, r;
	FILE *f;
	struct stat s;

	r = stat(file, &s);
	if (r == -1)
		return 0;

	f = fopen(file, "r");
	if (!f) {
		xerr("Cannot open conf file (%s): %s!\n",
			file, strerror(errno));
		return err;
	}

	while (1) {
		char buf[128], *q, *s, *val;
		unsigned short len;

		q = fgets(buf, sizeof(buf), f);
		if (!q) {
			err = 0;
			log_conf();
			break;
		}

		if ((*buf == '#') || (*buf == '\r') || (*buf == '\n'))
			continue;

		s = strchr(buf, '\r');
		if (s)
			*s = '\0';
		s = strchr(buf, '\n');
		if (s)
			*s = '\0';

		s = strchr(buf, '=');
		if (!s) {
			xerr("Invalid conf line [%s]!\n", buf);
			break;
		}
		*s = '\0';
		len = strlen(buf);
		while (strchr("\r\n\t", buf[len - 1])) {
			buf[len - 1] = '\0';
			len--;
		}

		val = s + 1;
		while (*val == ' ')
			val++;
		len = strlen(val);
		while (strchr("\r\n\t", val[len - 1])) {
			val[len - 1] = '\0';
			len--;
		}

		//xlog("DEBUG: var=[%s] value=[%s]\n", buf, val);

		if (strcmp(buf, "queue") == 0) {
			cfg_queue = strtoul(val, NULL, 0);
			continue;
		}

		if (strcmp(buf, "bad entries") == 0) {
			cfg_bad_entries = strtoul(val, NULL, 0);
			continue;
		}

		if (strcmp(buf, "sock") == 0) {
			cfg_sock_path = strdup(val);
			continue;
		}

		if (strcmp(buf, "grant time") == 0) {
			cfg_grant_time = strtoul(val, NULL, 0);
			continue;
		}

		if (strcmp(buf, "min pass len") == 0) {
			cfg_min_pass_len = strtoul(val, NULL, 0);
			continue;
		}

		if (strcmp(buf, "nfmark") == 0) {
			cfg_nfmark = strtoul(val, NULL, 0);
			continue;
		}

		if (strcmp(buf, "ctmark") == 0) {
			cfg_ctmark = strtoul(val, NULL, 0);
			continue;
		}

		xerr("Invalid conf option: %s\n", buf);
	}
	fclose(f);

	return err;
}
